from . import download
download.downloading()

import os
import platform
import shutil


rope_dll_folder = os.path.dirname(os.path.realpath(__file__))
dllw = os.path.join(rope_dll_folder, "rope_pyd")        
if platform.system() == "Windows":   
    if not os.path.exists(os.path.join(rope_dll_folder, "rope.pyd")):
        shutil.copyfile(dllw, dllw.replace("rope_pyd", "rope.pyd"))  

import bpy
from .handler import *
from .wire_props import *
from .wire_utils import *
from .interface import *
from .engine import *
from . import translate



debugDispla=[None]


def update_category(self, context):
    if len(self.category.strip()):

        for panel in panels:
            try:
                bpy.utils.unregister_class(panel)
            except Exception as e:
                print("GR preferences can not unregister: "%panel)
        
            panel.bl_category = self.category
            bpy.utils.register_class(panel)


class GravityRopePrefs(bpy.types.AddonPreferences):
    # addons customizable props
    #depthCurve = bpy.context.preferences.addons[__name__].preferences.depthCurve
    
    bl_idname = __name__
    # depthCurve : bpy.props.FloatProperty(default=1.0)

    category : bpy.props.StringProperty(
        description="Choose a name for the category of the panel",
        default="Gravity Rope", 
        update=update_category)

    # here you specify how they are drawn
    def draw(self, context):
        layout = self.layout
        # layout.prop(self, "depthCurve")

        layout.prop(self, "category")


panels = [
    GWIRECREATE_PT_view3d_panel,
    GWIREPOSE_PT_view3d_panel,
    
    GWIREDEFAULTSET_PT_view3d_panel,
    CUTS_PT_objectList,
    PENDUL_PT_objectList,
    GWIRESETTINGS_PT_view3d_panel,
    GW_ENGINE_PT_view3d_panel,
    GWIRE_PT_geometry,
    GWIRE_PT_collider,
    GWIRE_PT_bake,
    GWIRE_PT_geometry_curve,

    # pendulums
    GWIRE_PT_pendulumtool,
    GWIRE_PT_geotool,
    GWIRE_PT_cutstool,
    GWIRE_PT_display,
    ]


classes = [
    # rope solver updating
    RopeSimulateModal,

    GravityRopePrefs,
    GWireGetReadyOperator,
    GWanimatePinsOperator,
    AddGWireOperator,
    RemoveGWireOperator,
    AddHookOperator,
    RemoveHookOperator,
    updateFrameGWireOperator,
    HookPosIdxOperator,
    gravityWireOptimizeOperator,
    gravityWireCopySettOperator,
    bakeGravityWireGeoOperator,
    updateGWireBunchOperator,
    displayViewOperator,

    gw_options,
    Gravity_Wire_Preset_Liste,
    GW_collider_type,

    
    # pendulums
    PENDUL_UL_items,
    CUTS_UL_items,
    CUTS_OT_Randomize,
    PENDUL_OT_Randomize,
    PENDUL_OT_Uniforme,
    PENDUL_OT_actions,
    CUTS_OT_actions,
    PENDUL_OT_RemoveKeys,
    PENDUL_OT_addViewportSelection,
    PENDUL_OT_clearList,
    PENDUL_objectCollection,
    CUTS_objectCollection
    ] + panels # HELLO PANEL HERE


def register():
    from bpy.utils import register_class
    for cls in classes:
        try:
            register_class(cls)
        except Exception as e:
            print(cls)
    
    debugDispla[0] = bpy.types.SpaceView3D.draw_handler_add(
                   draw_debug_callback,(),
                   'WINDOW', 'POST_VIEW')

    register_props_oscene(GRAVITY_MASTER, GRAVITY_WIRE)
    register_engine_props()

    print("GRAVITY ROPE V%s READY"%str(bpy.app.version))
    translate.register()

def unregister():
    
    from bpy.utils import unregister_class
    for cls in reversed(classes):
        try:
            unregister_class(cls)
        except Exception as e:
            print(cls)

    # clear the post frame handler
    # blender registration stuf
    removeFramePostHandler("GRAVITY_MASTER")
    my_handler_list = bpy.app.handlers.load_post# alias
    fin = len(my_handler_list)

    for idx, func in enumerate(reversed(my_handler_list)):
        if func.__name__.startswith("GRAVITY_WIRE") :
            my_handler_list.pop(fin-1-idx) # remove old version

    unregister_props_oscene()
    unregister_engine_props()
    freeRopeModule()

    if bool(debugDispla):
        bpy.types.SpaceView3D.draw_handler_remove(debugDispla[0], 'WINDOW')
    translate.unregister()
        
if __name__ == "__main__":
    register()